import pygame as pg
import pygame.locals as pgl
import numpy as np
import OpenGL.GL as gl
import OpenGL.GLU as glu

class Figure:
    def __init__(self, width, height, pltname="VF plot"):
        self.display = (width, height)
        self.plots = []
        self.pltname = pltname

    def rotate(self, rot = (0.0, 0.0, 0.0)):
        gl.glRotate(rot[0], 1, 0, 0)
        gl.glRotate(rot[1], 0, 1, 0)
        gl.glRotate(rot[2], 0, 0, 1)

    def axes(self):
        gl.glLineWidth(2.0)
        gl.glBegin(gl.GL_LINES)

        # x-axis
        gl.glColor3f(235/255, 59/255, 59/255)
        gl.glVertex3f(0.0, 0.0, 0.0)
        gl.glVertex3f(0.2, 0.0, 0.0)

        # y-axis
        gl.glColor3f(145/255, 212/255, 97/255)
        gl.glVertex3f(0.0, 0.0, 0.0)
        gl.glVertex3f(0.0, 0.2, 0.0)

        # z-axis
        gl.glColor3f(147/255, 95/255, 184/255)
        gl.glVertex3f(0.0, 0.0, 0.0)
        gl.glVertex3f(0.0, 0.0, 0.2)

        gl.glEnd()

    def add(self, plot):
        self.plots.append(plot)

    def show(self):
        pg.init()
        pg.display.set_caption(self.pltname)
        pg.display.set_mode(self.display, pgl.DOUBLEBUF | pgl.OPENGL)
        gl.glOrtho(-1, 1, -1, 1, -100, 100)

        gl.glEnable(gl.GL_DEPTH_TEST);

        self.crot = [0, 0, 0]
        self.rotate(self.crot)
        rdrag = False
        while True:
            for event in pg.event.get():
                if event.type == pg.QUIT:
                    pg.quit()
                    quit()

                elif event.type == pg.MOUSEBUTTONDOWN:
                    if event.button == 1:
                        rdrag = True
                        rx, ry = event.pos

                elif event.type == pg.MOUSEBUTTONUP:
                    if event.button == 1:
                        rdrag = False

                elif event.type == pg.MOUSEMOTION:
                    if rdrag:
                        nx, ny = event.pos
                        dx, dy = (nx - rx, ny - ry)
                        self.crot[1] += dx/500

                        a, b, c = self.crot
                        self.rotate((0, dx/500, 0))

                        cz = np.sin(np.radians(b)) * (dy/500)
                        cx = np.cos(np.radians(b)) * (dy/500)
                        self.rotate((cx, 0, cz))
                        self.crot = [a + cx, b, c + cz]

            gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)
            self.axes()
            for plot in self.plots:
                plot.render()
            pg.display.flip()
            pg.time.wait(10)



"""
    A simple, time-varying vector field plot.

    u(x,y,z,t) represents the i-th component of the vector,
    v(x,y,z,t) represents the j-th component of the vector,
    w(x,y,z,t) represents the k-th component of the vector,
    thus,

        V(x,y,z,t) = u(x,y,z,t)i + v(x,y,z,t)j + w(x,y,z,t)k,

    where i, j, k are the unit vectors to the xyz coordinate system.
"""

# necessary imports
from vis3dvf.plot import Figure
from vis3dvf.vectorfield import *
import numpy as np

# i-th component of the time varying vector field
def u(x,y,z,t):
    return 0*x                                   # function must use at leas one of the x,y,z variables (due to numpy)

# j-th component of the time varying vector field
def v(x,y,z,t):
    return 0.1*np.sin(t + x)

# k-th component of the time varying vector field
def w(x,y,z,t):
    return 0*x                                   # function must use at leas one of the x,y,z variables

# Figure(window width, window height)
fig = Figure(800, 800, "Vector Field")

# VectorFieldT(u, v, w, vector density, initial time, final time, time delta)
fig.add(VectorFieldT(u, v, w, 6, 0, 2*np.pi, 0.05))

# Render field
fig.show()



import numpy as np
import OpenGL.GL as gl
import OpenGL.GLU as glu

class VectorField:
    def __init__(self, u, v, w, d, sf):

        space = np.linspace(-0.5,0.5,d)
        x, y, z = np.meshgrid(space, space, space)

        self.v = u(x*sf, y*sf, -z*sf)
        self.u = v(x*sf, y*sf, -z*sf)
        self.w = w(x*sf, y*sf, -z*sf)
        self.m = np.sqrt(self.u**2 + self.v**2 + self.w**2)
        self.m = self.m / np.max(self.m)
        self.x = y
        self.y = x
        self.z = -z

    def render(self):
        gl.glLineWidth(2.0)
        gl.glBegin(gl.GL_LINES)
        for i in range(self.x.shape[0]):
            for j in range(self.x.shape[1]):
                for k in range(self.x.shape[2]):
                    gl.glColor3f(self.m[i,j,k], 0.5, 1 - self.m[i,j,k])
                    gl.glVertex3f(self.x[i,j,k], self.y[i,j,k], self.z[i,j,k])
                    gl.glVertex3f(self.x[i,j,k] + self.u[i,j,k], self.y[i,j,k] + self.v[i,j,k], self.z[i,j,k] + self.w[i,j,k])
        gl.glEnd()

        gl.glPointSize(4.0)
        gl.glBegin(gl.GL_POINTS)
        for i in range(self.x.shape[0]):
            for j in range(self.x.shape[1]):
                for k in range(self.x.shape[2]):
                    gl.glColor3f(self.m[i,j,k], 0.5, 1 - self.m[i,j,k])
                    gl.glVertex3f(self.x[i,j,k] + self.u[i,j,k], self.y[i,j,k] + self.v[i,j,k], self.z[i,j,k] + self.w[i,j,k])

        gl.glEnd()

class VectorFieldT:
    def __init__(self, u, v, w, d, ti, tf, dt, sf):

        space = np.linspace(-0.5,0.5,d)
        # x, y, z = np.meshgrid(space, space, space)
        x, y, z = np.meshgrid(space, space, np.linspace(0,0,d))

        self.u = u
        self.v = v
        self.w = w

        self.x = x
        self.y = y
        self.z = z

        self.sf = sf

        self.t = ti
        self.T = (ti, tf, dt)

    def render(self):
        u = self.u(self.x*self.sf, self.y*self.sf, self.z*self.sf, self.t)
        v = self.v(self.x*self.sf, self.y*self.sf, self.z*self.sf, self.t)
        w = self.w(self.x*self.sf, self.y*self.sf, self.z*self.sf, self.t)
        m = np.sqrt(u**2 + v**2 + w**2)
        if np.max(m) != 0:
            m = m / np.max(m)

        gl.glLineWidth(2.0)
        gl.glBegin(gl.GL_LINES)
        for i in range(self.x.shape[0]):
            for j in range(self.x.shape[1]):
                for k in range(self.x.shape[2]):
                    gl.glColor3f(m[i,j,k], 0.5, 1 - m[i,j,k])
                    gl.glVertex3f(self.x[i,j,k], self.y[i,j,k], self.z[i,j,k])
                    gl.glVertex3f(self.x[i,j,k] + u[i,j,k], self.y[i,j,k] + v[i,j,k], self.z[i,j,k] + w[i,j,k])
        gl.glEnd()

        gl.glPointSize(4.0)
        gl.glBegin(gl.GL_POINTS)
        for i in range(self.x.shape[0]):
            for j in range(self.x.shape[1]):
                for k in range(self.x.shape[2]):
                    gl.glColor3f(m[i,j,k], 0.5, 1 - m[i,j,k])
                    gl.glVertex3f(self.x[i,j,k] + u[i,j,k], self.y[i,j,k] + v[i,j,k], self.z[i,j,k] + w[i,j,k])

        gl.glEnd()

        if self.t < self.T[1]:
            self.t += self.T[2]
        else:
            self.t = self.T[0]



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="vis3dvf",
    version="1.0.2",
    description="Derivative of vis3dpy, aimed at visualizing ℝ³ vector fields ",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/LukasDrsman/vis3dvf",
    author="Lukáš Dršman",
    author_email="lukaskodr@gmail.com",
    license="Public Domain",
    classifiers=[
        "License :: Public Domain",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["vis3dvf"],
    include_package_data=True,
    install_requires=["numpy", "pygame", "PyOpenGL", "PyOpenGL_accelerate"],
    entry_points={},
)
